#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

{
    'name': 'test_view_model',
    'version': '1.0',
    'depends': ['base', 'test_view_model'],
    'auto_install': False,
    'installable': True
}
